/**
 *  File name: "compare_entries.h"
 *  This file contains two classes used to sort enrties in the class 'Table'.
 *
 *  Created on 25-Apr-2006
 * 
 *  Copyright (C) 2005-2006 the MAGIC team.
 *  Author(s): Rani Zand <zanood@gmail.com>.
 *
 *
 *  This file is part of MAGIC, an Integrative and Accurate Comparative
 *  Genome Mapping tool.
 *
 *
 *  MAGIC is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GCC; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#ifndef _COMPARE_ENTRIES_H
#define _COMPARE_ENTRIES_H

#include "entry.h"

/**
 * Class name: compare_entries_1
 *
 * This class is used to sort entries according to the first organism. 
 *
 * @author Rani Zand
 *
 */
template <class GENOME_TYPE>
class compare_entries_1 {
public:
	bool operator() (const Entry<GENOME_TYPE> & entry1, const Entry<GENOME_TYPE> & entry2) const {
		GENOME_TYPE s1,e1,s2,e2;
		entry1.GetS1E1(s1,e1);
		entry2.GetS1E1(s2,e2);

        if( (s1 < s2) || ((s1==s2) && (e1<e2))  ) return true;

		if ( (s1==s2) && (e1==e2) ) {
			entry1.GetS2E2(s1,e1);
			entry2.GetS2E2(s2,e2);

			if( (s1 < s2) || ((s1==s2) && (e1<e2))  ) return true;
		}

		return false; 
	}
};

/**
 * Class name: compare_entries_2
 *
 * This class is used to sort entries according to the second organism. 
 *
 * @author Rani Zand
 *
 */
template <class GENOME_TYPE>
class compare_entries_2 {
public:
	bool operator() (const Entry<GENOME_TYPE> & entry1, const Entry<GENOME_TYPE> & entry2) const {
		GENOME_TYPE s1,e1,s2,e2;
		entry1.GetS2E2(s1,e1);
		entry2.GetS2E2(s2,e2);

        if( (s1 < s2) || ((s1==s2) && (e1<e2))  ) return true;

		if ( (s1==s2) && (e1==e2) ) {
			entry1.GetS1E1(s1,e1);
			entry2.GetS1E1(s2,e2);

			if( (s1 < s2) || ((s1==s2) && (e1<e2))  ) return true;
		}

		return false; 
	}
};


#endif
