/**
 *  File name: "mappingPhase.cpp"
 *  This file runs the mapping phase throw the main function with the 
 *  needed arguments.
 *
 *  Created on 25-Apr-2006
 * 
 *  Copyright (C) 2005-2006 the MAGIC team.
 *  Author(s): Rani Zand <zanood@gmail.com>.
 *
 *
 *  This file is part of MAGIC, an Integrative and Accurate Comparative
 *  Genome Mapping tool.
 *
 *
 *  MAGIC is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GCC; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 */


#include <stdio.h>
#include "table.h"
#include <iostream>
#include <fstream>
#include <string>
using namespace std;

#define MAX_NUM_OF_ARGS 18

int TraceOutput(int i, const char* outputTracePath);


int main(int argc, char* argv[])
{	
	if ( (argc != MAX_NUM_OF_ARGS-4) && (argc != MAX_NUM_OF_ARGS) )
	{
		cout << "\n Wrong arguments entered. \n Arguments should be: \n tableFile  numOfRowsInTable  outputFile  outputTracePath  cleanISPerc  cleanProPerc  cleanMinLen  dupPerc  orthPerc  orag1DataFile  org2DataFile deletedParalogOrg1 deletedParalogOrg1 [TEFile1]  [TEFile2]  [ProphagesFile1]  [ProphagesFile2]" << endl;
		return 0;
	}
	ifstream fd,fd2,fd3,fd4,fd5,fd6,fd7;
	ofstream print,sorted1,sorted2;
	
	fd.open(argv[1]);
	if ( !fd.is_open() ) {
		cout << "failed to open table file " << endl;
		return 1;
	}
	fd6.open(argv[10]);
	if ( !fd6.is_open() ) {
		cout << "failed to open first organism .prop file " << endl;
		return 2;
	}
	fd7.open(argv[11]);
	if ( !fd7.is_open() ) {
		cout << "failed to open second organism .prop file " << endl;
		return 3;
	}

	Table<unsigned long> test(fd, atol(argv[2]), atol(argv[7]), atof(argv[8]), fd6, fd7 );
	
	fd.close();
	fd6.close();
	fd7.close();

	TraceOutput(0, argv[4]);

	
//**************** Dup **********************	

	test.CleanDup();
	test.CleanDeleted();

//***************** short **********************

	test.CleanShortEntries();
	test.CleanDeleted();

	TraceOutput(10, argv[4]);
	
//******************** TE  ********************
if ( argc > MAX_NUM_OF_ARGS-4 )
{

	fd2.open(argv[14]);
	if ( !fd2.is_open() ) {
		cout << "failed to open TE org1 file " << endl;
		return 4;
	}
	test.CleanTE(fd2,1,atof(argv[5]));
	fd2.close();

	TraceOutput(20, argv[4]);

	fd3.open(argv[15]);
	if ( !fd3.is_open() ) {
		cout << "failed to open TE org2 file " << endl;
		return 5;
	}
	test.CleanTE(fd3,2,atof(argv[5]));
	fd3.close();

	test.CleanDeleted();

	TraceOutput(30, argv[4]);	
		
//******************** Prophages ************************

	fd4.open(argv[16]);
	if ( !fd4.is_open() ) {
		cout << "failed to open Prophages org1 file " << endl;
		return 6;
	}
	test.CleanProphages(fd4,1,atof(argv[6]));
	fd4.close();

	TraceOutput(40, argv[4]);

	fd5.open(argv[17]);
	if ( !fd5.is_open() ) {
		cout << "failed to open Prophages org2 file " << endl;
		return 7;
	}
	test.CleanProphages(fd5,2,atof(argv[6]));
	fd5.close();

	TraceOutput(50, argv[4]);		
}

//****************************************************
	
	test.CleanDeleted();
	test.IterativeRemoveOverlaps();

	TraceOutput(60, argv[4]);	

	test.CleanDeleted();
	test.Loop(atof(argv[9]), argv[12], argv[13]);	

	TraceOutput(70, argv[4]);

	test.CleanDeleted();
	test.RemoveFinalConflicts();

	TraceOutput(80, argv[4]);

	test.CleanDeleted();
	test.Clustering();

	TraceOutput(90, argv[4]);

	test.CleanDeleted();
	test.RemoveFinalOverlaping();

	test.CleanDeleted();

	TraceOutput(100, argv[4]);	

	const char *outputFile = argv[3];	
	print.open(outputFile);
	if ( !print.is_open() ) {
		cout << "failed to open the file " << endl;
		return 8;
	}
	test.PrintTable(print,1);
	print.close();

	return 0;
}


int TraceOutput(int i, const char* outputTracePath)
{
	ofstream fd;
	int timer = 0;	

	fd.open(outputTracePath);
	while ( ( !fd.is_open() ) && ( timer != 9999 ) )
	{		
		for ( int i=0 ; i < 1000 ; i++ ) ;
		timer ++ ;
		fd.open(outputTracePath);
	}
	if ( timer == 9999 )
	{
		cout << "failed to open trace file " << endl;
		return 9;
	}
	else // opened file
	{
		cout << i << endl;
		fd << i ;
		fd.close();
	}

	return 0;
}


