/**
 *  File name: "genome.h"
 *  This file contains the class Genome which is used to hold the 
 *  genome's informaton.
 *
 *  Created on 25-Apr-2006
 * 
 *  Copyright (C) 2005-2006 the MAGIC team.
 *  Author(s): Rani Zand <zanood@gmail.com>.
 *
 *
 *  This file is part of MAGIC, an Integrative and Accurate Comparative
 *  Genome Mapping tool.
 *
 *
 *  MAGIC is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GCC; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 59 Temple Place - Suite 330,
 *  Boston, MA 02111-1307, USA.
 */


#ifndef GENUM_H
#define GENUM_H

/*************************************************************/
/*********************** Class Genome *************************/
/*************************************************************/

#include <string>

using namespace std;

/**
 * Class name: Genome
 *
 * This class holds the information of a genome, which are:
 * 1) The genome's name.
 * 2) The genome's length.
 * 3) An indicator of the genum's circularity.
 *
 * @author Rani Zand
 *
 */
template<class GENOME_TYPE>
class Genome {

private: 
	/**
	 * The length of the genome in in nucleotides.
	 */
	GENOME_TYPE GenomeLen;

	/**
	 * equals 'true' if the genome is circular, 'false' if not.
	 */
	bool Circular;

	/**
	 * The name of the genome.
	 */
	string GenomeName;

public:

	/**
	 * Constructor.
	 */
	Genome() {}

	/**
	 * Distructor.
	 */
	~Genome() {}

	/**
	 * Return the length of the genome in nucleotides.
	 * @param 
	 * @return the length of the genome.
	 */
	GENOME_TYPE GetGenomeLen() const ;

	/**
	 * Sets the genome's length in nucleotides.
	 * @param len the new genome's length.
	 */
	void SetGenomeLen(GENOME_TYPE len);

	/**
	 * Return true if the genome is circular.
	 * @return true if the genome is circular.
	 */
	bool IsGenomeCircular() const;

	/**
	 * Sets the genome's circularity.
	 * @param circ the new genome's circularity.
	 */
	void SetGenomeCircular(bool circ);

	/**
	 * Return the genome's name.
	 * @return the genome's name.
	 */
	string GetGenomeName() const;

	/**
	 * Sets the genome's name.
	 * @param name the new genome's name.
	 */
	void SetGenomeName(const string name);

	/**
	 * operator =, copys all field to the given Genome .
	 */
	Genome& operator=(const Genome& gen);
};



/*************************************************************/
/******************** Class Genome Mothods ********************/
/*************************************************************/

template<class GENOME_TYPE>
Genome<GENOME_TYPE>& Genome<GENOME_TYPE>::operator=(const Genome& gen) {
	this->GenomeLen = gen.GenomeLen;
	this->GenomeName = gen.GenomeName;
	this->Circular = gen.Circular;
	
	return *this;
}

/*************************************************************/

template<class GENOME_TYPE>
GENOME_TYPE Genome<GENOME_TYPE>::GetGenomeLen() const{
	return this->GenomeLen;
}


/*************************************************************/

template<class GENOME_TYPE>
void Genome<GENOME_TYPE>::SetGenomeLen(const GENOME_TYPE len) {
	this->GenomeLen = len;
}

/*************************************************************/

template<class GENOME_TYPE>
string Genome<GENOME_TYPE>::GetGenomeName() const{
	string tmp = this->GenomeName;
	
	return tmp;
}

/*************************************************************/

template<class GENOME_TYPE>
void Genome<GENOME_TYPE>::SetGenomeName(const string name) {
	this->GenomeName = name;	
}

/*************************************************************/

template<class GENOME_TYPE>
bool Genome<GENOME_TYPE>::IsGenomeCircular() const {
	return this->Circular;
}

/*************************************************************/

template<class GENOME_TYPE>
void Genome<GENOME_TYPE>::SetGenomeCircular(const bool circ) {
	this->Circular = circ ;
}




#endif
